defmodule PrymnWeb.UserForgotPasswordLive do
  use PrymnWeb, :live_view

  alias Prymn.Accounts

  @impl true
  def render(assigns) do
    ~H"""
    <div class="mx-auto max-w-sm">
      <.header class="mb-10 text-center">
        Forgot your password?
        <:subtitle>
          Enter your email that your account was registered with and we will
          send a password reset link to your inbox.
        </:subtitle>
      </.header>

      <.simple_form for={@form} id="reset_password_form" phx-submit="send_email">
        <.input
          field={@form[:email]}
          type="email"
          label="Email"
          placeholder="Enter your email address"
          required
        />
        <:actions>
          <Button.primary phx-disable-with="Sending..." class="w-full">
            Send password reset instructions
          </Button.primary>
        </:actions>
      </.simple_form>

      <div class="mt-4 flex justify-center divide-x text-sm">
        <.link href={~p"/auth/register"} class="px-3">Register</.link>
        <.link href={~p"/auth/log_in"} class="px-3">Log in</.link>
      </div>
    </div>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, assign(socket, form: to_form(%{}, as: "user"))}
  end

  @impl true
  def handle_event("send_email", %{"user" => %{"email" => email}}, socket) do
    if user = Accounts.get_user_by_email(email) do
      Accounts.deliver_user_reset_password_instructions(
        user,
        &url(~p"/auth/reset_password/#{&1}")
      )
    end

    info =
      "If your email is in our system, you will receive instructions to reset your password shortly."

    {:noreply,
     socket
     |> put_flash(:info, info)
     |> redirect(to: ~p"/")}
  end
end
